@@ -64,9 +64,9 @@ module IcdGenerator
                           "cl_kernel"        => "CL_INVALID_KERNEL",
                           "cl_event"         => "CL_INVALID_EVENT",
                           "cl_sampler"       => "CL_INVALID_SAMPLER"}
-  $non_standard_error = [ "clGetExtensionFunctionAddressForPlatform" ]
+  $non_standard_error = [ "clGetExtensionFunctionAddressForPlatform", "clSVMAlloc" ]
   $versions_entries = []
-  $buff=20
+  $buff=50
   $license = <<EOF
 Copyright (c) 2012, Brice Videau <brice.videau@imag.fr>
 Copyright (c) 2012, Vincent Danjean <Vincent.Danjean@ens-lyon.org>
@@ -313,6 +313,7 @@ EOF
     ocl_icd_header += "#  pragma GCC diagnostic ignored \"-Wcpp\"\n"
     ocl_icd_header += "#  define CL_USE_DEPRECATED_OPENCL_1_0_APIS\n"
     ocl_icd_header += "#  define CL_USE_DEPRECATED_OPENCL_1_1_APIS\n"
+    ocl_icd_header += "#  define CL_USE_DEPRECATED_OPENCL_1_2_APIS\n"
     ocl_icd_header += "#  include <CL/opencl.h>\n"
     ocl_icd_header += self.include_headers
     ocl_icd_header += "#pragma GCC diagnostic pop\n"
@@ -323,11 +324,25 @@ EOF
 
 struct _cl_icd_dispatch {
 EOF
+    nb=0
     $api_entries_array.each { |entry|
+      version = entry.split("\n").
+	grep(/ CL_API_SUFFIX__(VERSION_[0-9_]+)[^0-9_]/).join('').
+	gsub(/.* CL_API_SUFFIX__(VERSION_[0-9_]+)[^0-9_].*$/, '\1')
+      if (version != '') then
+	ocl_icd_header += '#ifdef CL_'+version+"\n"
+      end
       ocl_icd_header += entry.gsub("\r","").
 	sub(/CL_API_CALL\n?(.*?)\(/m,'(CL_API_CALL*\1)('+"\n  ").
 	gsub(/\) (CL_API_SUFFIX__VERSION)/m,"\n) \\1").gsub(/\s*$/,'').
-	gsub(/^[\t ]+/,"    ").gsub(/^([^\t ])/, '  \1') + "\n\n"
+	gsub(/^[\t ]+/,"    ").gsub(/^([^\t ])/, '  \1') + "\n"
+      if (version != '') then
+	ocl_icd_header += '#else'+"\n"
+	ocl_icd_header += '  CL_API_ENTRY cl_int (CL_API_CALL* clUnknown'+nb.to_s+")(void);\n"
+	ocl_icd_header += '#endif'+"\n"
+      end
+      ocl_icd_header += "\n"
+      nb=nb+1
     }
     ocl_icd_header += "};\n"
     ocl_icd_header += "#endif\n\n"
@@ -397,7 +412,7 @@ EOF
       end
     }
     if( $known_entries[$api_entries.length+$buff-1] ) then
-      ocl_icd_bindings_source += "  #{$known_entries[i]}\n"
+      ocl_icd_bindings_source += "  #{$known_entries[$api_entries.length+$buff-1]}\n"
     else
       ocl_icd_bindings_source += "  (void *) NULL\n"
     end
@@ -508,10 +523,11 @@ EOF
     $api_entries.each { |func_name, entry|
       next if skip_funcs.include?(func_name)
       clean_entry = entry.sub(/(.*\)).*/m,'\1').gsub("/*","").gsub("*/","").gsub("\r","") + "{\n"
+      return_type = entry.match(/CL_API_ENTRY (.*) CL_API_CALL/)[1]
       parameters = clean_entry.match(/\(.*\)/m)[0][1..-2]
       parameters.gsub!(/\[.*?\]/,"")
-      parameters.sub!(/\(.*?\*\s*(.*?)\)\s*\(.*?\)/,'\1')
-      ocl_icd_loader_gen_source += clean_entry.gsub(/\[.*?\]/,"")
+      parameters.sub!(/\(.*?\*\s*(.*?)\)\s*\(.*?\)/m,'\1')
+      ocl_icd_loader_gen_source += clean_entry.gsub(/\*\[.*?\]/,"*  ").gsub(/\[.+?\]/,"")
       first_parameter = parameters.match(/.*?\,/m)
       if not first_parameter then
         first_parameter =  parameters.match(/.*/m)[0]
@@ -531,11 +547,23 @@ EOF
           ocl_icd_loader_gen_source += "    if( errcode_ret != NULL ) {\n";
           ocl_icd_loader_gen_source += "      *errcode_ret = #{$cl_data_type_error[fps[0]]};\n"
           ocl_icd_loader_gen_source += "    }\n"
-          ocl_icd_loader_gen_source += "    RETURN(NULL);\n"
+          if return_type != "void" then
+            ocl_icd_loader_gen_source += "    RETURN(NULL);\n"
+          else
+            ocl_icd_loader_gen_source += "    return;\n"
+          end
         elsif ($non_standard_error.include?(func_name)) then
-          ocl_icd_loader_gen_source += "    RETURN(NULL);\n"
+          if return_type != "void" then
+            ocl_icd_loader_gen_source += "    RETURN(NULL);\n"
+          else
+            ocl_icd_loader_gen_source += "    return;\n"
+          end
         else
-          ocl_icd_loader_gen_source += "    RETURN(#{$cl_data_type_error[fps[0]]});\n"
+          if return_type != "void" then
+            ocl_icd_loader_gen_source += "    RETURN(#{$cl_data_type_error[fps[0]]});\n" if return_type != "void"
+          else
+            ocl_icd_loader_gen_source += "    return;\n"
+          end
         end
       }
        
@@ -545,9 +573,13 @@ EOF
       ocl_icd_loader_gen_source += "  if( (struct _#{fps[0]} *)#{fps[1]} == NULL) {\n"
       error_handler.call
       ocl_icd_loader_gen_source += "  }\n"
-      ocl_icd_loader_gen_source += "  RETURN(((struct _#{fps[0]} *)#{fps[1]})->dispatch->#{func_name}("
-      ocl_icd_loader_gen_source += ps.join(", ")
-      ocl_icd_loader_gen_source += "));\n"
+      if return_type != "void" then
+        ocl_icd_loader_gen_source += "  RETURN(((struct _#{fps[0]} *)#{fps[1]})->dispatch->#{func_name}("
+        ocl_icd_loader_gen_source += ps.join(", ")
+        ocl_icd_loader_gen_source += "));\n"
+      else
+        ocl_icd_loader_gen_source += "  return;"
+      end
       ocl_icd_loader_gen_source += "}\n\n"
     }
     ocl_icd_loader_gen_source += "#pragma GCC visibility push(hidden)\n\n"
