@@ -231,7 +231,7 @@ static inline unsigned int _open_driver(unsigned int num_icds,
     RETURN(num_icds);
   }
 
-  lib_path_length = strlen(lib_path);
+  lib_path_length = strnlen(lib_path, lib_path_length);
   
   if( lib_path[lib_path_length-1] == '\n' )
     lib_path[lib_path_length-1] = '\0';
@@ -447,7 +447,7 @@ static void __initClIcd( void ) {
   debug_init();
   cl_uint num_icds = 0;
   int is_dir = 0;
-  DIR *dir;
+  DIR *dir = NULL;
   const char* dir_path=getenv("OCL_ICD_VENDORS");
   if (! dir_path || dir_path[0]==0) {
     debug(D_DUMP, "OCL_ICD_VENDORS empty or not defined, using %s", ETC_OPENCL_VENDORS);
@@ -519,6 +519,10 @@ static void __initClIcd( void ) {
     _icds = (struct vendor_icd*)realloc(_icds, _num_icds * sizeof(struct vendor_icd));
   }
   debug(D_WARN, "%d valid vendor(s)!", _num_icds);
+
+  if (dir != NULL){
+    closedir(dir);
+  }
   return;
  abort:
   _num_icds = 0;
@@ -526,6 +530,9 @@ static void __initClIcd( void ) {
     free(_icds);
     _icds = NULL;
   }
+  if (dir != NULL){
+    closedir(dir);
+  }
   return;
 }
 
